# ---------------------------------------------------------------------------
# BBSLIST.MPS: Download BBS List MPE for Mystic BBS
# ---------------------------------------------------------------------------
# This MPE allows user's to download a text file containing all the
# BBSes contained in the BBS list.
#
# Fully commented and written by Brother James.  Sysop of:
#
# Sonic-Boom BBS (Mystic BBS HQ)
# Node 1: (717) 784-8706
# ---------------------------------------------------------------------------
# START OF PROGRAM (BBSLIST.MPS):

# Define program variables:

Var DatFile String           # Holds Path/FileName of BBSLIST data file
Var DesFile String           # Holds Path/FileName of BBSLIST comment file
Var OutFile String           # Holds Path/FileName of BBSLIST output file
                             # Note that this variable uses the node TEMP
                             # directory to make sure that user's connected
                             # to other nodes can also download a BBS list
                             # at the same time without conflict.

Var Total   Integer          # Total number of BBS's listed in text file.
Var A       Byte             # Temp variable used in for/loop.
Var B       Byte             # Temp variable used to read comments.
Var Str     String           # Temp variable used to read comments.

# Since the MPE doesn't have functions to access the BBSLIST data files,
# we have to open them and use the MPE engine's file I/O functions.
# The following variables mimic the BBSLISTREC for the BBSLIST.DAT as
# defined in the RECORDS.PAS file structures for Mystic BBS:

Var BBSDeleted Boolean       # Is record deleted?
Var BBSName    String        # Name of BBS
Var BBSSysop   String        # Sysop Name
Var BBSBaud    LongInt       # Baud Rate
Var BBSPhone   String        # Phone Number
Var BBSCity    String        # BBS Location
Var BBSWare    String        # BBS Software
Var BBSAdded   String        # Added by (User Name)
Var BBSComment Boolean       # Is there a comment for this entry?
Var BBSPointer LongInt       # Pointer to extended comment
Var BBSLines   Byte          # Number of lines in extended comment.

GetCFG                       # Load CFG variables

# Assign program variables:

DatFile := CfgDataPath + 'BBSLIST.DAT'
DesFile := CfgDataPath + 'BBSLIST.DES'
OutFile := CfgSysPath  + 'TEMP' + Int2Str(NodeNum) + '\BBSLIST.TXT'

# Check to see if BBSLIST.DAT exists.  If not, exit program.

If Not FExist(DatFile)
     WriteLn ('~CR~12There are no entries in the BBS list.')
     Halt
EndIf

# Prompt user to download BBS list, exit program if "NO".

If Not InputYN ('~CR~12Download the BBS list? ~11')
     Halt
EndIf

Write ('~CR~14Compiling BBS list: ')

FOpen (1, Bin,  Reset,   DatFile)      # Open BBSLIST.DAT as binary file.
FOpen (2, Text, ReWrite, OutFile)      # Create output file as text file. 

# Write header in output file.  You can change this to personalize it for
# your BBS if you want to.

FWriteLn (2, '')
FWriteLn (2, 'BBS list created on ', DateStr(DateTime), ' at ', TimeStr(DateTime), '.')
FWriteLn (2, '')
FWriteLn (2, '-----------------------------------------------------------------------')

# Set total number of BBS's in list to zero.

Total := 0

# Here we use the FReadRec command to read in BBSLIST records.  FReadRec is
# a really primative record system, but it DOES (along with FRead) allow
# us to access to every single data file which Mystic BBS uses!

While Not Eof(1)                       # Repeat until End of File.

     FReadRec (1, BBSDeleted)          # Use FReadRec to read in a single
     FReadRec (1, BBSName,  40)        # BBSLISTREC record.
     FReadRec (1, BBSSysop, 30)
     FReadRec (1, BBSBaud)
     FReadRec (1, BBSPhone, 12)
     FReadRec (1, BBSCity,  25)
     FReadRec (1, BBSWare,  10)
     FReadRec (1, BBSAdded, 30)
     FReadRec (1, BBSComment)
     FReadRec (1, BBSPointer)
     FReadRec (1, BBSLines)

     # Here is where we write the record to the text file.  This can also
     # be modified if you don't like the structure of the bbs list.

     If Not BBSDeleted  # If record not marked as deleted, write it to file.
          FWriteLn (2, '     BBS Name: ', BBSName)
          FWriteLn (2, '        Sysop: ', BBSSysop)
          FWriteLn (2, 'Max Baud Rate: ', Int2Str(BBSBaud))
          FWriteLn (2, ' Phone Number: ', BBSPhone)
          FWriteLn (2, '     Location: ', BBSCity)
          FWriteLn (2, ' BBS Software: ', BBSWare)

          If BBSComment      # If there is a comment, write it to file.
               FWriteLn (2, '     Comments:')
               FWriteLn (2, '')

               FOpen (3, Bin, Reset, DesFile)  # Open comment file
               FSeek (3, BBSPointer)           # Seek to starting position
               For A := 1 to BBSLines          # Read each line in comment
                    FRead (3, B, 1)
                    FRead (3, Str, B)
                    FWriteLn (2, Str)          # Write comment line to file
               FEnd
               FClose (3)                      # Close comment file.
          EndIf

          # This line separates bbs list entries.

          FWriteLn (2, '-----------------------------------------------------------------------')

          Total := Total + 1           # Increase number of BBS's in list.
     EndIf
WEnd

# Here we write the footer of the BBS list.  Again, it is easily changable
# if you don't like the format.

FWriteLn (2, '')
FWriteLn (2, '* Total number of BBSes in list: ', Int2Str(Total))

FClose (1)                             # Close BBSLIST.DAT file.
FClose (2)                             # Close output file.

WriteLn ('Done.')

# If local login, allow the BBS list to be saved to a file on the drive.
# Otherwise, allow the user to download the file.

If Local
     If InputYN ('~CR~12Local mode: Save BBS list to file? ~11')
          Write ('~CR~03Enter full path and filename for BBS list~CR~09:')
          Str := CfgSysPath + 'BBSLIST.TXT'                              
          Str := Input (60, 12, Str)
          If Str <> ''
               Write ('~CR~14Saving: ~15', Str, '~14: ')
               If FCopy(OutFile, Str)
                    WriteLn ('OK.')
               Else
                    WriteLn ('~12ERROR.')
               EndIf
          EndIf
     EndIf
Else                                 
     MenuCmd ('F3', OutFile)          
EndIf

FErase  (OutFile)                      # Erase BBS list from temp directory.

# END OF PROGRAM (BBSLIST.MPS).
